
# q-expansion of newform 567.2.p.c, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 567
weight = 2
poly_data = [9, -21, 64, -1, 64, -19, 34, -4, 7, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[29709, -15165, 44484, -75410, 36059, -19883, 7882, -4381, 1348, -339], 72795], [[98232, 130790, 30472, 47450, 51907, 25351, 10326, 4352, 2394, 658], 72795], [[-398583, 81750, -414508, -2560, -118353, -85319, -20279, -21678, -796, -4192], 218385], [[-336546, 31515, 82264, 361225, -150966, 233822, -28438, 54399, -9272, 8236], 218385], [[-54156, 166780, 72109, 175205, -42836, 104352, -8823, 21759, -2962, 3301], 72795], [[11514, -51320, -30640, -51488, -3072, -27923, -998, -5659, 248, -840], 14559], [[17856, 106440, 124546, 111910, 5736, 84554, 1181, 17808, -1373, 3085], 43677], [[178101, -668685, -222374, -1019855, 295851, -601717, 65033, -128469, 18997, -18476], 218385]]

hecke_ring_character_values = [[407, [-1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [325, [1, 0, 0, 0, 0, 0, 1, 0, 0, 0]]]
aps_data = [[0, 0, 0, -1, -1, -1, 0, -1, -1, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 1, 0, 0, 1, -1, 0, 0, 1, 0], [1, -1, 0, 1, 1, 1, 1, 0, -1, -1], [-2, 1, 1, 0, 1, 0, -1, 1, 1, 0], [0, -2, 1, 1, 0, 1, 0, 0, -1, 1], [0, 0, -1, 2, 2, 0, 3, 1, 0, -1], [0, 0, 0, 3, 4, 2, 0, 3, 2, -2], [-1, 1, -2, 2, 3, 1, 1, 2, 1, -2], [-1, 2, -1, 4, 5, -1, -2, 0, 4, -4], [0, 2, 0, 0, 0, -1, 0, 1, 1, -1], [2, 0, 2, 0, 2, 0, 2, 0, 0, 2], [0, 0, -3, 0, -4, -4, 0, 0, 0, 0], [-2, 0, 2, -1, -4, -3, 0, -2, -1, 1], [3, 2, 1, 0, 1, 0, 3, 0, 0, 1], [-2, 2, 2, 0, 1, 0, -1, 1, 1, 0], [0, 2, 0, 0, 3, -1, 3, 0, 2, -3], [-1, 0, -2, 4, 3, 3, 1, 4, 3, 0], [0, 0, -1, 2, -1, 3, 2, 1, 0, 2], [0, 2, -1, -5, -2, -3, 0, 0, 2, -2], [0, 1, 1, 0, -3, 0, 0, -3, -3, 0], [4, -3, 2, -4, -1, 3, 4, 4, -3, 2], [3, 0, -6, 0, -2, -2, 0, 0, 0, 0], [6, -3, 0, 2, 1, -1, 6, -2, 1, 0], [0, 6, -3, 2, 7, -5, 0, 0, 6, -6], [0, -8, 7, -2, -5, 1, 6, -1, -2, 4], [4, 1, 6, 2, 6, 6, -4, 2, 6, 0], [0, -1, -1, -1, -3, -2, 0, -1, -2, 1], [0, -3, 2, -2, -4, 0, -5, -1, -2, 3], [1, -10, 5, 4, -2, 6, 2, 0, -4, 4], [-1, 0, -3, -1, -2, -1, 0, -2, -2, 0], [0, -2, -2, -1, 1, -3, 0, 1, 3, -2], [-6, -5, 1, 0, 2, 2, -3, -2, 0, 4], [1, 6, -3, 1, 7, -6, 2, 0, 4, -4], [-3, -5, 4, 2, -3, 1, 3, 2, 1, 4], [0, 0, 0, 10, 10, 5, 4, 5, 5, -5], [-4, -4, -9, 0, -2, -2, -2, 1, 0, -3], [1, -12, 0, -1, -5, 5, 1, 1, -5, 0], [12, 0, 6, 0, 5, 5, 0, 0, 0, 0], [9, 2, -4, -1, -2, -2, 9, 1, 2, -4], [-10, 2, 5, 0, -4, -1, -5, -8, -3, 4], [-3, 2, -1, 0, 5, -5, -6, 0, 1, -1], [0, 3, -3, -1, 2, -1, 0, -1, -1, -3], [0, 3, -5, 10, 12, 1, 1, 5, 3, -7], [1, -4, 2, 4, 3, 1, 2, 0, -3, 3], [-4, 5, -4, 0, -1, -8, -2, 0, 7, -1], [1, 0, -4, -4, -10, -6, 0, -8, -7, 1], [-2, -4, 2, -10, -7, -3, -4, 0, -7, 7], [0, 4, -3, 2, 10, -6, -3, 1, 2, -9], [-3, -3, 2, -4, -9, -3, 3, -4, -3, 6], [-4, 2, -1, -4, -6, 1, 4, -4, 1, 7], [1, 14, -7, 4, 10, -6, 2, 0, 5, -5], [2, -5, -6, 0, -1, 2, 1, 2, -3, -3], [9, 0, -2, 2, 3, 1, 0, 4, -4, -8], [9, 1, 0, 6, -2, 2, 9, -6, -2, 0], [-8, -2, -2, 0, -2, 6, -4, 4, -8, -6], [0, 3, 1, 12, 17, 5, 3, 6, 10, -11], [2, 1, -14, -3, -12, -6, -2, -3, -6, 6], [0, 12, -5, -12, -4, -7, -1, -6, 1, -2], [3, -10, 5, 1, -7, 8, 6, 0, -8, 8], [-2, 2, 3, 0, 11, 3, -1, 13, 8, -2], [0, 0, -5, 5, 11, 6, 0, 10, 8, -2], [-2, 2, -1, 3, 10, -7, -4, 0, 5, -5], [0, 8, -2, 8, 15, 3, -12, 4, 10, -11], [-2, -4, 0, 13, 12, 6, 2, 13, 6, -6], [-4, 0, -3, 14, 17, 8, 4, 14, 8, -9], [-13, 6, -11, 1, -4, -7, -13, -1, 7, -11], [7, 0, -1, -7, -16, -9, 0, -14, -4, 10], [-12, 0, -12, 0, -8, -3, -6, 1, -5, -9], [2, -8, 4, -7, -6, -1, 4, 0, -2, 2], [0, -2, 2, 6, -2, 11, -9, 3, 3, 5], [-2, -1, 2, -4, -1, -7, 2, -4, -7, -6], [4, 0, 2, 0, 2, -7, 2, -7, 9, 9], [1, -3, 7, 0, 2, 5, 1, 0, -5, 7], [-2, 0, 11, 0, 10, 10, 0, 0, -2, -2], [6, 1, 0, -9, -5, 5, 6, 9, -5, 0], [8, 1, 13, 0, 5, 2, 4, -5, 3, 10], [-1, 7, -2, -9, -3, -3, 1, -9, -3, 0], [-2, 1, -2, -13, -19, -7, 2, -13, -7, 12], [16, -1, 6, 0, 8, 6, 8, 7, 2, 1], [-3, 0, -11, -7, -19, -12, 0, -14, -13, 1], [-13, 0, 6, -2, -4, -2, 0, -4, 2, 6], [2, -10, -1, 0, 2, 5, 1, -2, -3, 4], [8, -4, 2, -12, -3, -9, 16, 0, -3, 3], [-5, 3, -8, -8, -5, -13, 5, -8, -13, -8], [-1, 9, -6, -10, -4, -4, 1, -10, -4, 0], [-3, -10, 5, -2, -8, 6, -6, 0, -1, 1], [-1, -3, 4, 3, 0, 4, -1, -3, -4, 4], [-6, 0, -10, -5, -14, -9, 0, -10, -11, -1], [-7, 0, 9, -5, -4, 1, 0, -10, -13, -3], [6, 6, 3, -7, -5, 8, 6, 7, -8, 3], [0, -8, 5, -12, -11, -10, -12, -6, -9, 5], [0, -9, 5, 0, -9, 5, -5, 0, -4, 9], [-1, -10, 5, -5, -10, 5, -2, 0, -5, 5], [5, 9, -3, 5, -4, 1, 5, -5, -1, -3], [12, 0, -9, 0, 6, 6, 0, 0, -3, -3], [18, -2, 6, 3, 4, 2, 18, -3, -2, 6], [0, -15, 10, 6, -10, 14, -9, 3, -2, 13], [1, -8, 16, 9, 10, 8, -1, 9, 8, -2], [-11, 21, -15, -4, 1, -16, -11, 4, 16, -15], [20, 0, -9, 1, -1, -2, 0, 2, -4, -6], [-2, 5, -19, 0, -20, -15, -1, -11, -5, -9], [0, 5, -6, -18, -21, -7, -15, -9, -10, 12], [-11, -6, -6, -16, -35, -15, 11, -16, -15, 20], [0, 0, -1, -24, -26, -11, -4, -12, -13, 14], [2, 6, -6, 0, -6, -9, 1, -3, 3, -3], [18, 0, 4, 8, 25, 17, 0, 16, 5, -11], [15, -24, 6, 5, -11, 17, 15, -5, -17, 6], [12, 3, 12, 0, 4, 6, 6, 1, -2, 3], [-4, 12, -6, -1, -1, 0, -8, 0, -7, 7], [-13, 2, -5, 11, 15, 6, 13, 11, 6, -9], [0, 3, -6, 8, 10, -1, -1, 4, 1, -6], [-10, -16, 8, -8, -10, 2, -20, 0, -8, 8], [-10, 8, -5, 0, 3, -5, -5, 11, 8, -8], [-1, 0, -12, 3, -3, -6, 0, 6, -3, -9], [24, -2, -8, 0, -22, -7, 12, -23, -15, 1], [7, -18, 9, 1, -2, 3, 14, 0, -8, 8], [0, 12, -11, -12, -10, -7, -24, -6, -5, 4], [-6, 1, 13, -4, 9, -2, 6, -4, -2, -11], [0, 14, -7, 4, 14, -10, 0, 0, 13, -13], [-4, 3, 1, 0, -4, -6, -2, -8, 2, 4], [-5, 0, 8, -10, -19, -9, 0, -20, -13, 7], [-3, 14, -7, -10, 7, -14, -3, 10, 14, -7], [18, -4, 2, 0, 14, 11, 9, 19, 3, -5], [6, -9, 16, -8, -22, 4, -6, -8, 4, 26], [-9, 20, -10, 4, 10, -6, -18, 0, 14, -14], [-1, -18, 7, 6, -6, 13, -1, -6, -13, 7], [-12, 24, -11, -2, 10, -21, -12, 2, 21, -11], [6, 12, -6, 8, 13, -5, 12, 0, 12, -12], [16, 1, 0, -13, -18, -6, -16, -13, -6, 12], [0, 6, 0, 2, 10, -1, -1, 1, 7, -9], [0, -4, 2, 4, -4, 8, 0, 0, -5, 5], [14, 6, 0, 1, 6, -6, 14, -1, 6, 0], [-23, 0, 14, -2, 0, 2, 0, -4, -3, 1], [18, -17, 10, 8, -2, 12, 18, -8, -12, 10], [-7, -6, 3, 22, 12, 10, -14, 0, -1, 1], [0, 9, -8, -12, -1, -16, -15, -6, -5, -5], [10, -5, -10, 0, -5, 1, 5, 1, -6, -6], [-15, 0, 21, 0, 11, 11, 0, 0, -6, -6], [24, 8, -1, 0, 2, -10, 12, 1, 12, 1], [18, -2, 1, 6, 1, 5, 36, 0, 2, -2], [-5, -6, -9, 26, 22, 9, 5, 26, 9, -13], [0, -9, 14, -6, -2, -2, 2, -3, 2, -1], [7, -10, 5, 4, 0, 4, 14, 0, -12, 12], [-18, -4, -13, 0, -2, 5, -9, 12, -7, -14], [-6, 0, 2, -2, -8, -6, 0, -4, -2, 2], [1, -24, 12, -1, 0, -1, 2, 0, 1, -1], [0, 1, -15, -10, -14, -15, 3, -5, -19, 9], [-8, 3, -4, 2, 2, 4, 8, 2, 4, 2], [-11, 2, -10, 8, 11, -1, 11, 8, -1, -12], [4, -12, 4, -3, 8, -4, 4, 3, 4, 4], [-6, 0, 14, 1, 15, 14, 0, 2, 7, 5], [22, 0, -10, -12, -23, -11, 0, -24, -17, 7], [-15, 4, -2, -10, 4, -6, -15, 10, 6, -2], [0, -12, 6, 4, -14, 14, 4, 2, -4, 16], [5, -10, 5, 4, 8, -4, 10, 0, -5, 5], [11, -3, 5, 12, 8, -3, 11, -12, 3, 5], [18, 11, -5, 3, -8, 3, 18, -3, -3, -5], [-19, 6, -3, -9, -8, -1, -38, 0, -1, 1], [0, -10, 0, 30, 30, 5, 3, 15, 5, -15], [-1, 3, 15, -13, -1, -4, 1, -13, -4, -3], [-10, 8, -4, -5, -5, 0, -20, 0, 2, -2], [-1, 0, 0, -1, -11, -10, 0, -2, 4, 6], [-3, 9, -1, 15, -8, 7, -3, -15, -7, -1], [8, -9, -12, 0, 1, 3, 4, 7, -2, -6], [0, 1, -3, -16, -20, -6, -27, -8, -10, 12], [0, 27, -10, -6, 3, 5, 7, -3, 14, -6], [-12, 6, -4, 0, -18, -17, -6, -25, -1, 7]]
